// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"fmt"
	"os"
	"path"
	"strings"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"

	"xbus.io/go-xbus/v3"
	//	"xbus.io/go-xbus/v3/api"
	fullenv "xbus.io/xbus/fullenv_tests"
	"xbus.io/xbus/utils/license"
	"xbus.io/xbus/version"
)

// Logger is a logger automatically configured by command line options
var Logger xbus.Logger

var (
	v = viper.New()
)

func usage(cmd *cobra.Command) {
	if err := cmd.Usage(); err != nil {
		Logger.Errorf("Error printing usage: %s", err)
	}
	os.Exit(1)
}

// RootCmd represents the base command when called without any subcommands
var RootCmd = &cobra.Command{
	Use:   "xbus-fullenv",
	Short: "Xbus fullenv tests utility",
	Long: `Xbus fullenv tests utility

xbus-fullenv is a utility for setting up a full xbus environment from
a single configuration file.`,
	Run: version.Cmd,
}

// Execute adds all child commands to the root command sets flags appropriately.
// This is called by main.main(). It only needs to happen once to the rootCmd.
func Execute() {
	if err := RootCmd.Execute(); err != nil {
		fmt.Println(err)
		os.Exit(-1)
	}
}

func must(err error) {
	if err != nil {
		panic(err)
	}
}

func init() {
	cobra.OnInitialize(initConfig)

	flags := RootCmd.PersistentFlags()
	version.InitFlags(flags)

	flags.BoolP("debug", "d", false, "Enable debug mode")
	flags.Bool("trace", false, "Enable trace mode")
	flags.String("dsn", "", "Postgres dsn")
	flags.String("target", "", "Target directory (default to a directory in /tmp)")

	flags.StringSlice("var", []string{}, "Configuration variable value (KEY=value)")

	must(v.BindPFlag("logging.debug", flags.Lookup("debug")))
	must(v.BindPFlag("logging.trace", flags.Lookup("trace")))
	must(v.BindPFlag("target", flags.Lookup("target")))
	must(v.BindPFlag("var", flags.Lookup("var")))
}

// initConfig reads in config file and ENV variables if set.
// it also setup the logger
func initConfig() {
	v.SetEnvPrefix("XBUS_FULLENV")
	v.AutomaticEnv() // read in environment variables that match

	Logger = xbus.GetLogger(&xbus.LoggerOptions{
		Debug: v.GetBool("logging.debug"),
		Trace: v.GetBool("logging.trace"),
	})
	license.SetupLicenseWatcher(Logger)
}

func loadOptions(cmd *cobra.Command, filename string) (string, fullenv.Options, error) {

	options, err := fullenv.OptionsFromFile(filename)
	if err != nil {
		Logger.Errorf("Could not load file: %s", err)
		return "", fullenv.Options{}, err
	}

	if cmd.Flag("dsn").Changed {
		options.Dsn = cmd.Flag("dsn").Value.String()
	}

	if cmd.Flag("target").Changed {
		options.TargetDir = cmd.Flag("target").Value.String()
	}

	if v.IsSet("var") {
		m := make(map[string]string)
		for _, entry := range v.GetStringSlice("var") {
			splitted := strings.SplitN(entry, "=", 2)
			if len(splitted) != 2 {
				Logger.Errorf("Invalid 'var' entry: '%s'. Expects 'KEY=value'", entry)
				return "", fullenv.Options{}, err
			}
			m[splitted[0]] = splitted[1]
		}
		options.Vars = m
	}

	options.Logger = Logger
	basename := path.Base(filename)
	basename = basename[:len(basename)-len(path.Ext(basename))]

	return basename, options, nil
}
