// Copyright 2016 Orus.io. All rights reserved.

package sentinel

import (
	"context"

	"xbus.io/go-xbus/v3/api"
	"xbus.io/xbus/services"
	"xbus.io/xbus/storage"
)

const (
	// ServiceName name of the service
	ServiceName = "sentinel"

	// QueueName queue name for queuesubscribes
	QueueName = "xbus.services." + ServiceName
)

// Options is used to create a 'processstate' service
type Options struct {
	services.Options

	ActorStorage storage.ActorStorage
}

// NewService creates a Service
func (o Options) NewService() *Service {
	return &Service{
		BaseService: services.NewBaseService(ServiceName, o.Options),
		o:           o,
	}
}

// Service is in charge of emitting process status events to various subjects
type Service struct {
	services.BaseService

	o       Options
	handler *api.SentinelHandler
}

// Startup starts the service
func (s *Service) Startup() error {
	sentinelHandler := api.NewSentinelHandler(
		context.Background(), s.Conn, sentinelServer{s},
	)
	if err := s.SubscribeAll(
		s.HandlerQueueSubscribe(sentinelHandler, QueueName),
	); err != nil {
		return err
	}
	s.handler = sentinelHandler

	// Ping all actors so they report their status if online
	// XXX Actors that do not respond within 5 seconds may be forced offline
	// but it is less backward compatible. We may do it later.
	if err := s.handler.SurveyPublish(api.SurveyRequest{OnlineStatus: true}); err != nil {
		s.o.Options.Logger.Errorf("Failed to send the actor survey: %s", err)
	}
	return nil
}

type sentinelServer struct {
	*Service
}

func (s sentinelServer) ActorReady(ctx context.Context, actorID string) {
	actorUUID, err := api.UUIDFromString(actorID)
	if err != nil {
		s.Log.Errorf("Received invalid actorID: %s (err: %s)", actorID, err)
	}
	// signal the actor is online
	if err := s.o.ActorStorage.SetOnline(actorUUID, true); err != nil {
		s.Log.Errorf("Error calling SetOnline: %s", err)
	}
}

func (s sentinelServer) ActorLeaving(ctx context.Context, actorID string) {
	actorUUID, err := api.UUIDFromString(actorID)
	if err != nil {
		s.Log.Errorf("Received invalid actorID: %s (err: %s)", actorID, err)
	}
	// signal the actor is online
	if err := s.o.ActorStorage.SetOnline(actorUUID, false); err != nil {
		s.Log.Errorf("Error calling SetOnline: %s", err)
	}
}

// ListActors returns the list of actors.
//
// When returning, we know that the
// service finished any ongoing discovery of the actors statuses.
// The initial implementation is way too naïve, but it
// is not a big deal until we go really distributed (mainly because of the
// director service that rely on it to properly start).
func (s sentinelServer) ListActors(
	ctx context.Context, send func(api.ClientGetActorsReply),
) error {
	actors := s.o.ActorStorage.List()

	batchSize := 100

	for batchSize < len(actors) {
		send(api.ClientGetActorsReply{
			Actors: actors[0:batchSize:batchSize],
		})
		actors = actors[batchSize:]
	}
	send(api.ClientGetActorsReply{
		Actors: actors,
	})
	return nil
}
